/*
This file is part of the Juju GUI, which lets users view and manage Juju
environments within a graphical interface (https://launchpad.net/juju-gui).
Copyright (C) 2015 Canonical Ltd.

This program is free software: you can redistribute it and/or modify it under
the terms of the GNU Affero General Public License version 3, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranties of MERCHANTABILITY,
SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero
General Public License for more details.

You should have received a copy of the GNU Affero General Public License along
with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

'use strict';

YUI.add('user-profile-entity', function () {

  juju.components.UserProfileEntity = React.createClass({
    displayName: 'UserProfileEntity',

    propTypes: {
      acl: React.PropTypes.object,
      changeState: React.PropTypes.func,
      children: React.PropTypes.oneOfType([React.PropTypes.object, React.PropTypes.array]),
      displayConfirmation: React.PropTypes.func,
      entity: React.PropTypes.object.isRequired,
      expanded: React.PropTypes.bool,
      getDiagramURL: React.PropTypes.func,
      permission: React.PropTypes.string,
      switchModel: React.PropTypes.func,
      type: React.PropTypes.string.isRequired
    },

    /**
      Calls to switch the env to the one the user clicked on.
       @method _switchModel
      @param {Object} model The model to switch to, with these attributes:
        - name: the model name;
        - id: the model unique identifier;
        - owner: the user owning the model, like "admin" or "who@external".
      @param {Object} evt The click event.
    */
    _switchModel: function (model, evt) {
      this.props.switchModel(model);
    },

    /**
      Navigate to the entity details.
       @method _viewEntity
      @param {String} id The entity id.
      @param {Object} evt The click event.
    */
    _viewEntity: function (id, evt) {
      const url = window.jujulib.URL.fromLegacyString(id);
      this.props.changeState({ profile: null, store: url.path() });
    },

    /**
      Handle clicks on tags.
       @method _handleTagClick
      @param {Object} e The event.
      @param {String} tag The tag to view.
    */
    _handleTagClick: function (tag, e) {
      this.props.changeState({
        profile: null,
        search: {
          tags: tag
        }
      });
    },

    /**
      Generate a list of tags.
       @method _generateTags
      @returns {Object} A list of tag components.
    */
    _generateTags: function () {
      const tags = [];
      const entity = this.props.entity;
      const tagList = entity.tags;
      if (!tagList) {
        return;
      }
      tagList.forEach(tag => {
        tags.push(React.createElement(
          'li',
          { className: 'user-profile__comma-item link',
            key: entity.id + '-' + tag,
            onClick: this._handleTagClick.bind(this, tag),
            role: 'button',
            tabIndex: '0' },
          tag
        ));
      });
      return React.createElement(
        'div',
        { className: 'twelve-col no-margin-bottom' },
        React.createElement(
          'div',
          { className: 'two-col' },
          'Tags'
        ),
        React.createElement(
          'ul',
          { className: 'ten-col last-col' },
          tags
        )
      );
    },

    /**
      Generate a list of commits.
       @method _generateTags
      @returns {Object} A list of commits.
    */
    _generateCommits: function () {
      const commits = [];
      const entity = this.props.entity;
      const commitList = entity.commits;
      if (!commitList) {
        return;
      }
      commitList.forEach(commit => {
        commits.push(React.createElement(
          'li',
          { key: entity.id + '-' + commit.id },
          React.createElement(
            'div',
            { className: 'seven-col' },
            commit.description
          ),
          React.createElement(
            'div',
            { className: 'three-col' },
            commit.time
          ),
          React.createElement(
            'div',
            { className: 'two-col last-col' },
            commit.author
          )
        ));
      });
      return React.createElement(
        'div',
        { className: 'user-profile__entity-commits twelve-col no-margin-bottom' },
        React.createElement(
          'div',
          { className: 'twelve-col' },
          'Latest commits',
          React.createElement(
            'span',
            { className: 'user-profile__size' },
            '(',
            commitList.length,
            ' commits)'
          )
        ),
        React.createElement(
          'ul',
          { className: 'user-profile__entity-commit-list twelve-col no-margin-bottom' },
          commits
        )
      );
    },

    /**
      Generate the diagram for a bundle.
       @method _generateDiagram
      @return {Object} The diagram.
    */
    _generateDiagram: function () {
      if (this.props.type !== 'bundle') {
        return;
      }
      const url = this.props.getDiagramURL(this.props.entity.id);
      return React.createElement(
        'div',
        { className: 'user-profile__entity-diagram twelve-col' },
        React.createElement('object', { type: 'image/svg+xml', data: url,
          className: 'entity-content__diagram-image' })
      );
    },

    /**
      Generate the description.
       @method _generateDescription
      @return {Object} The description.
    */
    _generateDescription: function () {
      const description = this.props.entity.description;
      if (!description) {
        return;
      }
      return React.createElement(
        'div',
        { className: 'twelve-col no-margin-bottom' },
        React.createElement(
          'div',
          { className: 'two-col' },
          'Description'
        ),
        React.createElement(
          'div',
          { className: 'ten-col last-col' },
          description
        )
      );
    },

    /**
      Generate the series for a charm.
       @method _generateSeries
      @return {Object} The series component.
    */
    _generateSeries: function () {
      if (this.props.type !== 'charm') {
        return;
      }
      return React.createElement(
        'div',
        { className: 'twelve-col last-col' },
        'Series: ',
        this.props.entity.series
      );
    },

    /**
      Generate the services for a bundle.
       @method _generateServices
      @return {Object} The services component.
    */
    _generateServices: function () {
      if (this.props.type !== 'bundle') {
        return;
      }
      const entity = this.props.entity;
      const services = [];
      const applications = entity.applications || entity.services || {};
      Object.keys(applications).forEach(name => {
        services.push(React.createElement(
          'li',
          { className: 'user-profile__comma-item',
            key: entity.id + '-service-' + name },
          name
        ));
      });
      return React.createElement(
        'div',
        { className: 'twelve-col last-col' },
        'Composed of:',
        React.createElement(
          'ul',
          { className: 'user-profile__entity-service-list' },
          services
        )
      );
    },

    /**
      Generate the credentials for a model.
       @method _generateCredentials
      @return {Object} The credential markup.
    */
    _generateModelInfo: function () {
      if (this.props.type !== 'model') {
        return null;
      }
      const model = this.props.entity;
      // See the _generateRow function in model-list.js for matching logic.
      // Both sections should be kept roughly in sync.
      const region = model.region || 'no region';
      let owner = '--';
      if (model.owner) {
        owner = model.owner.split('@')[0];
      }
      // The main different between this and model-list.js is that we display
      // the credentials used here in place of the model name, as the model
      // name is displayed elsewhere in the expanded row.
      return React.createElement(
        'div',
        { className: 'modelInfo' },
        React.createElement(
          'div',
          { className: 'prepend-two two-col' },
          owner
        ),
        React.createElement(
          'div',
          { className: 'two-col' },
          model.numMachines
        ),
        React.createElement(
          'div',
          { className: 'two-col' },
          model.cloud + '/' + region
        ),
        React.createElement(
          'div',
          { className: 'two-col' },
          this.props.permission
        ),
        React.createElement(
          'div',
          { className: 'two-col last-col' },
          React.createElement(juju.components.DateDisplay, {
            date: model.lastConnection || '--',
            relative: true })
        )
      );
    },

    /**
      Generate and return the destroy button if the user is allowed to destroy
      the given model.
       @param {Object} model A model object as returned by the controller
        listModelsWithInfo API call.
      @return {Object} The react button node.
    */
    _generateDestroyButton: function (model) {
      const props = this.props;
      if (model.isController) {
        // Do not allow destroying the controller model.
        return null;
      }
      if (!props.acl) {
        // This should never happen.
        console.warn('acl object not available while listing models');
        return null;
      }
      if (!props.acl.canRemoveModel(model)) {
        // The user has not enough access to destroy a model.
        return null;
      }
      return React.createElement(juju.components.GenericButton, {
        action: props.displayConfirmation,
        type: 'inline-base',
        title: 'Destroy model'
      });
    },

    render: function () {
      const props = this.props;
      const entity = props.entity;
      const type = props.type;
      const isModel = type === 'model';
      const isCharm = type === 'charm';
      // Model names will be in the format "username/model-name" so we have to
      // extract the part we need.
      let title = React.createElement(
        'div',
        { className: 'entity-title' },
        React.createElement(
          'span',
          { className: 'entity-title__name' },
          entity.name
        )
      );
      let buttonAction;
      let destroyButton = null;
      if (isModel) {
        let name = entity.name;
        if (name.indexOf('/') !== -1) {
          name = name.split('/')[1];
        }
        title = React.createElement(
          'div',
          { className: 'entity-title' },
          React.createElement(
            'span',
            { className: 'entity-title__name' },
            name
          ),
          React.createElement(
            'span',
            { className: 'entity-title__credential' },
            entity.credentialName
          )
        );
        buttonAction = this._switchModel.bind(this, {
          id: entity.uuid,
          name: name,
          owner: entity.owner
        });
        destroyButton = this._generateDestroyButton(entity);
      } else {
        buttonAction = this._viewEntity.bind(this, entity.id);
      }
      const icon = isCharm ? React.createElement('img', { className: 'user-profile__entity-icon',
        src: entity.icon,
        title: entity.name }) : undefined;
      const classes = {
        'user-profile__entity': true,
        'user-profile__list-row': true
      };
      return React.createElement(
        juju.components.ExpandingRow,
        { classes: classes,
          expanded: props.expanded },
        props.children,
        React.createElement(
          'div',
          null,
          React.createElement(
            'div',
            { className: 'expanding-row__expanded-header twelve-col' },
            React.createElement(
              'div',
              { className: 'six-col no-margin-bottom' },
              icon,
              title
            ),
            React.createElement(
              'div',
              { className: 'expanding-row__expanded-header-action ' + 'six-col last-col no-margin-bottom' },
              destroyButton,
              React.createElement(juju.components.GenericButton, {
                action: buttonAction,
                type: 'inline-neutral',
                title: isModel ? 'Manage' : 'View' })
            )
          ),
          React.createElement(
            'div',
            { className: 'expanding-row__expanded-content twelve-col ' + 'no-margin-bottom' },
            this._generateSeries(),
            this._generateServices(),
            this._generateModelInfo(),
            this._generateDiagram(),
            this._generateDescription(),
            this._generateTags(),
            this._generateCommits()
          )
        )
      );
    }
  });
}, '', {
  requires: ['date-display', 'expanding-row', 'generic-button']
});